package com.calypso.bildroid;

import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.methods.HttpGet;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.http.AndroidHttpClient;
import android.os.AsyncTask;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;

public class ListAdapter extends BaseAdapter {

	/* VARIABLES */

	// a String constant to store the class file name that is used in Log
	// methods.
	private static final String TAG = "ListAdapter";

	// a context so we can later create a view within it
	private Context appContext;

	// store a cache of resized bitmaps
	private Bitmap[] cache;

	// store data which will be processed
	private Article[] data;

	private LoadImageTask imageGen;

	private LayoutInflater mInflater;

	/* CONSTRUCTOR */

	public ListAdapter(Context context, Article[] articleCollection,
			Object previousCache) {
		this.appContext = context;
		this.data = articleCollection;

		// obtains the LayoutInflater from the given context.
		this.mInflater = LayoutInflater.from(this.appContext);

		// get our LoadImageTask ready to execute
		imageGen = new LoadImageTask();

		if (previousCache != null) {
			// if there is a pre-existing data
			this.cache = (Bitmap[]) previousCache;
		} else {
			// if there isn't any pre-existing data
			this.cache = new Bitmap[this.data.length];
		}

		// executes the image processing task in the background
		imageGen.execute();
	}

	/* METHODS */

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.widget.Adapter#getCount()
	 */
	public int getCount() {
		return this.data.length;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.widget.Adapter#getItem(int)
	 */
	public Object getItem(int position) {
		return (this.data[position]);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.widget.Adapter#getItemId(int)
	 */
	public long getItemId(int position) {
		return position;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.widget.Adapter#getView(int, android.view.View,
	 * android.view.ViewGroup)
	 */
	public View getView(int position, View convertView, ViewGroup parent) {

		ViewHolder holder = new ViewHolder();

		if (convertView == null) {
			convertView = mInflater.inflate(R.layout.news_list_item, null);

			holder.text1 = (TextView) convertView
					.findViewById(R.id.newsListTitle);
			holder.text2 = (TextView) convertView
					.findViewById(R.id.newsListDescription);
			holder.image = (ImageView) convertView
					.findViewById(R.id.newsListImage);

			convertView.setTag(holder);
		} else {
			// recycle an old view (it might have old thumbs in it!)
			holder = (ViewHolder) convertView.getTag();
		}

		if (cache[position] == null) {
			holder.image.setImageResource(R.drawable.ic_launcher);
		} else {
			holder.image.setImageBitmap(cache[position]);
		}

		Article anArticle = this.data[position];

		holder.text1.setText(anArticle.getTitle());
		holder.text2.setText(anArticle.getContent().trim().substring(0, 75));

		return convertView;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.widget.BaseAdapter#isEmpty()
	 */
	@Override
	public boolean isEmpty() {
		return super.isEmpty();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.widget.BaseAdapter#notifyDataSetChanged()
	 */
	@Override
	public void notifyDataSetChanged() {
		super.notifyDataSetChanged();
	}

	/* INNER CLASSES */

	private class LoadImageTask extends AsyncTask<Void, Void, Void> {

		/* VARIABLES */

		/* METHODS */

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#doInBackground(Params[])
		 */
		@Override
		protected Void doInBackground(Void... params) {

			// iterate over all news
			for (int i = 0; i < data.length; i++) {

				// check whether the task is cancelled before or not
				if (isCancelled()) {
					return null;
				}

				// if this position have a Bitmap, don't process a new data
				if (cache[i] != null) {
					continue;
				}

				Article anArticle = data[i];

				cache[i] = downloadBitmap(anArticle.getImageURL());

				publishProgress(); // after this onProgressUpdate is invoked
			}

			return null;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPostExecute(java.lang.Object)
		 */
		@Override
		protected void onPostExecute(Void result) {
			super.onPostExecute(result);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onPreExecute()
		 */
		@Override
		protected void onPreExecute() {
			super.onPreExecute();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see android.os.AsyncTask#onProgressUpdate(Progress[])
		 */
		@Override
		protected void onProgressUpdate(Void... values) {
			super.onProgressUpdate(values);

			// notify the attached observers that the underlying data has been changed 
			notifyDataSetChanged();
		}

		private Bitmap downloadBitmap(String url) {
			if (url != null) {
				final AndroidHttpClient client = AndroidHttpClient
						.newInstance("Android");
				final HttpGet getRequest = new HttpGet(url);

				try {
					HttpResponse response = client.execute(getRequest);
					final int statusCode = response.getStatusLine()
							.getStatusCode();
					if (statusCode != HttpStatus.SC_OK) {
						Log.w(TAG, "Error " + statusCode
								+ " while retrieving bitmap from " + url);
						return null;
					}

					final HttpEntity entity = response.getEntity();
					if (entity != null) {
						InputStream inputStream = null;
						try {
							inputStream = entity.getContent();

							BitmapFactory.Options options = new BitmapFactory.Options();
							options.inSampleSize = 4;

							final Bitmap bitmap = BitmapFactory.decodeStream(
									new FlushedInputStream(inputStream), null,
									options);
							return bitmap;
						} finally {
							if (inputStream != null) {
								inputStream.close();
							}
							entity.consumeContent();
						}
					}
				} catch (IOException e) {
					getRequest.abort();
					Log.w(TAG, "I/O error while retrieving bitmap from " + url,
							e);
				} catch (IllegalStateException e) {
					getRequest.abort();
					Log.w(TAG, "Incorrect URL: " + url);
				} catch (Exception e) {
					getRequest.abort();
					Log.w(TAG, "Error while retrieving bitmap from " + url, e);
				} finally {
					if ((client instanceof AndroidHttpClient)) {
						((AndroidHttpClient) client).close();
					}
				}
			}
			return null;

		} // end of downloadBitmap(String url)

	} // end of LoadImageTask

	// a bug in the previous versions of BitmapFactory.decodeStream may prevent
	// this code from working over a slow connection.
	// FlushedInputStream(inputStream) instead to fix the problem
	private class FlushedInputStream extends FilterInputStream {
		public FlushedInputStream(InputStream inputStream) {
			super(inputStream);
		}

		@Override
		public long skip(long n) throws IOException {
			long totalBytesSkipped = 0L;
			while (totalBytesSkipped < n) {
				long bytesSkipped = in.skip(n - totalBytesSkipped);
				if (bytesSkipped == 0L) {
					int b = read();
					if (b < 0) {
						break; // we reached EOF
					} else {
						bytesSkipped = 1; // we read one byte
					}
				}
				totalBytesSkipped += bytesSkipped;
			}
			return totalBytesSkipped;
		}
	}

	/**
	 * ViewHolder is a class that will keep references to layout ids for faster
	 * performance / efficiency
	 * 
	 * http://developer.android.com/training/improving-layouts/smooth-scrolling.
	 * html#ViewHolder
	 * 
	 * @author Celal
	 * 
	 */
	private class ViewHolder {
		// A ViewHolder object stores each of the component views inside the tag
		// field of the Layout, so you can immediately access them without the
		// need to look them up repeatedly.
		TextView text1;
		TextView text2;
		ImageView image;
	}
}
