package com.calypso.bildroid;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

public class NewsDatabase {

	/* VARIABLES */

	private static final String TAG = "NewsDatabase";

	/**
	 * Defines the database name
	 */
	private static final String DATABASE_NAME = "news.db";

	/**
	 * Defines the database table name
	 */
	private static final String DATABASE_TABLE_NAME = "bilnews";

	/**
	 * Defines the database version
	 */
	private static final int DATABASE_VERSION = 1;

	// The columns we'll include in the bilnews table
	public static final String KEY_ROWID = "_id";
	public static final String KEY_TITLE = "title";
	public static final String KEY_CONTENT = "content";
	public static final String KEY_URL = "url";
	public static final String KEY_IMAGE_URL = "imageURL";
	public static final String[] allColumns = { KEY_TITLE, KEY_CONTENT,
		KEY_URL, KEY_IMAGE_URL };

	/**
	 * The CREATE TABLE statement is used to create a table in a database.
	 */
	private static final String DB_CREATE_STATEMENT = "CREATE TABLE "
			+ DATABASE_TABLE_NAME + " (_id INTEGER PRIMARY KEY AUTOINCREMENT, "
			+ KEY_TITLE + " TEXT NOT NULL, " + KEY_CONTENT + " TEXT NOT NULL, "
			+ KEY_URL + " TEXT NOT NULL, " + KEY_IMAGE_URL + " TEXT);";

	private final NewsDBOpenHelper databaseOpenHelper;
	private SQLiteDatabase db;

	/* CONSTRUCTOR */

	/**
	 * Constructor
	 * 
	 * @param context
	 *            - The Context within which to work, used to create the DB
	 */
	public NewsDatabase(Context context) {

		// instantiate a NewsDBOpenHelper class
		databaseOpenHelper = new NewsDBOpenHelper(context);
	}

	/* METHODS */

	/**
	 * void open() throws SQLException
	 * 
	 * @see android.database.sqlite.SQLiteOpenHelper.getWritableDatabase()
	 * @throws SQLException
	 */
	public void open() throws SQLException {
		// the SQLiteOpenHelper class (a parent of DatabaseHelper)
		// has a "getWritableDatabase" method that returns an
		// object of type SQLiteDatabase that represents an open
		// connection to the database we've opened (or created).
		this.db = this.databaseOpenHelper.getWritableDatabase();
	}

	/**
	 * void close()
	 * 
	 * Close any open database object.
	 */
	public void close() {
		this.databaseOpenHelper.close();
	}

	/**
	 * long insertNews(News aNews)
	 * 
	 * @param a
	 *            News object
	 * @return the row ID of the newly inserted row, or -1 if an error occurred
	 */
	public long insertArticle(Article anArticle) {
		// ContentValues is used to store a set of values
		ContentValues vals = new ContentValues();
		// get title & add it to the set.
		vals.put(KEY_TITLE, anArticle.getTitle());
		// get content & add it to the set.
		vals.put(KEY_CONTENT, anArticle.getContent());
		// get url & add it to the set.
		vals.put(KEY_URL, anArticle.getArticleURL());
		// get imageURL & add it to the set.
		vals.put(KEY_IMAGE_URL, anArticle.getImageURL());

		return (this.db.insert(DATABASE_TABLE_NAME, null, vals));
	}

	/**
	 * Cursor query(String[] columns, String selection, String[] selectionArgs,
	 * String groupBy, String having, String orderBy)
	 * 
	 * Query the table, returning a Cursor over the result set.
	 * 
	 * @param columns
	 *            - A list of which columns to return. Passing null will return
	 *            all columns, which is discouraged to prevent reading data from
	 *            storage that isn't going to be used.
	 * @param selection
	 *            - A filter declaring which rows to return, formatted as an SQL
	 *            WHERE clause (excluding the WHERE itself). Passing null will
	 *            return all rows for the given table.
	 * @param selectionArgs
	 *            - You may include ?s in selection, which will be replaced by
	 *            the values from selectionArgs, in order that they appear in
	 *            the selection. The values will be bound as Strings.
	 * @param groupBy
	 *            - A filter declaring how to group rows, formatted as an SQL
	 *            GROUP BY clause (excluding the GROUP BY itself). Passing null
	 *            will cause the rows to not be grouped
	 * @param having
	 *            - A filter declare which row groups to include in the cursor,
	 *            if row grouping is being used, formatted as an SQL HAVING
	 *            clause (excluding the HAVING itself). Passing null will cause
	 *            all row groups to be included, and is required when row
	 *            grouping is not being used.
	 * @param orderBy
	 *            - How to order the rows, formatted as an SQL ORDER BY clause
	 *            (excluding the ORDER BY itself). Passing null will use the
	 *            default sort order, which may be unordered.
	 * @return - A Cursor object, which is positioned before the first entry.
	 *         Note that Cursors are not synchronized, see the documentation for
	 *         more details.
	 */
	public Cursor query(String[] columns, String selection,
			String[] selectionArgs, String groupBy, String having,
			String orderBy) {

		// perform a database query
		// SQL Statement : SELECT * FROM news;
		Cursor cursor = db.query(DATABASE_TABLE_NAME, // table
				columns, // columns
				selection, // condition or selection
				selectionArgs, // condition or selection arguments
				groupBy, // groupBy
				having, // having
				orderBy // orderBy
				);

		if (cursor == null) {
			return null;
		} else if (!cursor.moveToFirst()) {
			cursor.close();
			return null;
		}
		return cursor;

	}

	public Article[] getAllArticles() {

		// perform a database query
		// SQL Statement : SELECT * FROM bilnews;
		Cursor cursor = query(allColumns, null, null, null, null, null);

		if (cursor != null) {
			Article[] articleCollection = new Article[cursor.getCount()];

			// move the cursor to the first row.
			cursor.moveToFirst();

			// while there is still a data to process
			while (!cursor.isAfterLast()) {
				// instantiate a News object & add it to collection
				articleCollection[cursor.getPosition()] = new Article(
						cursor.getString(0), cursor.getString(1),
						cursor.getString(2), cursor.getString(3));

				cursor.moveToNext();
			}

			// closes the Cursor, releasing all of its resources and making it
			// completely invalid
			cursor.close();

			return articleCollection;

		} else {

			return null;

		}

	}

	private static class NewsDBOpenHelper extends SQLiteOpenHelper {

		/**
		 * @param context
		 *            - to use to open or create the database
		 * @param name
		 *            - of the database file, or null for an in-memory database
		 * @param factory
		 *            - to use for creating cursor objects, or null for the
		 *            default
		 * @param version
		 *            - number of the database (starting at 1); if the database
		 *            is older, onUpgrade(SQLiteDatabase, int, int) will be used
		 *            to upgrade the database; if the database is newer,
		 *            onDowngrade(SQLiteDatabase, int, int) will be used to
		 *            downgrade the database
		 * 
		 * @see 
		 *      android.database.sqlite.SQLiteOpenHelper.SQLiteOpenHelper(Context
		 *      context, String name, CursorFactory factory, int version)
		 */
		public NewsDBOpenHelper(Context context) {
			super(context, DATABASE_NAME, null, DATABASE_VERSION);
		}

		/**
		 * void onCreate(SQLiteDatabase db)
		 * 
		 * Creates the underlying database with table name and column names
		 * taken from the NewsDatabase class.
		 * 
		 * @see android.database.sqlite.SQLiteOpenHelper#onCreate(android.database.sqlite.SQLiteDatabase)
		 */
		@Override
		public void onCreate(SQLiteDatabase db) {
			// called by the parent class when a DB doesn't exist

			// Send an INFO log message.
			Log.i(TAG, "onCreate(SQLiteDatabase db) is called.");

			try {
				// execute the CREATE TABLE statement
				db.execSQL(DB_CREATE_STATEMENT);
			} catch (SQLException e) {
				Log.e(TAG, "The SQL string is invalid.");
			}

		}

		/**
		 * void onCreate(SQLiteDatabase db)
		 * 
		 * Currently, the database is upgraded the database by destroying the
		 * existing data.
		 * 
		 * @see android.database.sqlite.SQLiteOpenHelper#onUpgrade(android.database.sqlite.SQLiteDatabase,
		 *      int, int)
		 */
		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
			// called by the parent when a DB needs to be upgraded

			// Send an INFO log message.
			Log.i(TAG,
					"onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) is called.");

			// remove the table and existing data
			db.execSQL("DROP TABLE IF EXISTS " + DATABASE_TABLE_NAME);

			// create a new one
			onCreate(db);

		}

	}
}
